<?php

namespace Drupal\views_extender_eca\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Entity\Eca;
use Drupal\eca_base\BaseEvents;
use Drupal\eca_base\Event\CustomEvent;
use Drupal\views_extender\Plugin\views\filter\EntityFieldCompare;
use Drupal\views_extender_eca\Service\MemoryState;

/**
 * Filter handler which token.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsFilter("entity_field_compare_eca")
 */
class EntityFieldCompareEca extends EntityFieldCompare {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['default_value'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  protected function valueForm(&$form, FormStateInterface $form_state) {
    $entities = Eca::loadMultiple();
    $options = [];
    foreach ($entities as $entity) {
      if (!$entity->status() || !str_starts_with($entity->id(), 'view_filter_')) {
        continue;
      }
      $options[$entity->id()] = $entity->label();
    }

    $form['value'] = [
      '#prefix'        => '<div class="clear"></div>',
      '#type'          => 'select',
      '#required'      => TRUE,
      '#title'         => $this->t('ECA Model'),
      '#empty_option'  => $this->t('- Select -'),
      '#empty_value'   => '',
      '#options'       => $options,
      '#default_value' => $this->value,
    ];

  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['default_value'] = [
      '#type'          => 'textfield',
      '#required'      => FALSE,
      '#title'         => $this->t('Default value'),
      '#description'   => $this->t('The default value when  ECA not return value. This field support token.'),
      '#default_value' => $this->options['default_value'],
    ];
    if ($this->moduleHandler->moduleExists('token')) {
      $form['token_tree'] = [
        '#type'            => 'container',
        '#theme'           => 'token_tree_link',
        '#token_types'     => 'all',
        '#global_types'    => TRUE,
        '#click_insert'    => TRUE,
        '#recursion_limit' => 3,
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function opEqual($expression) {
    $placeholder = $this->placeholder();
    $operator = $this->operator();
    $event = new CustomEvent($this->value, []);
    $this->eventDispatcher->dispatch($event, BaseEvents::CUSTOM);
    /** @var \Drupal\eca\EcaState $state */

    $value = MemoryState::registry($this->value);
    if (is_null($value)) {
      $value = $this->token->replace($this->options['default_value'], $this->getTokenData());
    }
    /* @phpstan-ignore-next-line */
    $this->query->addWhereExpression($this->options['group'], "$expression $operator $placeholder", [$placeholder => $value]);
  }

}
